@@ -1,6 +1,6 @@
 =head1 NAME
 
-Date::Tolkien::Shire.pm
+Date::Tolkien::Shire - Convert dates into the Shire Calendar.
 
 =head1 DESCRIPTION
 
@@ -20,9 +20,25 @@ am curious to see how this gets used (if it gets used that is).
 
 Tom Braun <tbraun@pobox.com>
 
-=head1 DATE
+Thomas R. Wyant, III F<wyant at cpan dot org>
 
-February 2001
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (c) 2001-2003, 2006 Tom Braun. All rights reserved.
+
+Copyright (C) 2017 Thomas R. Wyant, III
+
+The calendar implemented on this module was created by J.R.R. Tolkien,
+and the copyright is still held by his estate.  The license and
+copyright given herein applies only to this code and not to the
+calendar itself.
+
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
+
+This program is distributed in the hope that it will be useful, but
+without any warranty; without even the implied warranty of
+merchantability or fitness for a particular purpose.
 
 =cut
 
@@ -33,7 +49,7 @@ use Time::Local;
 
 use vars qw($VERSION $ERROR);
 
-$VERSION = 0.11;
+$VERSION = '1.19_01';
 
 =head1 METHOD REFERENCE
 
@@ -87,7 +103,7 @@ sub error {
 This method takes either the seconds from the start of the epoch (like what 
 time returns) or another shire date object, and sets the date of the 
 object in question equal to that date.  If the object previously contained
-a date, it will be overwritten.  Locatime, rather than utc, is used in 
+a date, it will be overwritten.  Localtime, rather than utc, is used in 
 converting from epoch date.
 
 Please see the note below on calculating the year if your curious how
@@ -97,7 +113,7 @@ I arrived by that.
 
 sub set_date {
     my ($self, $date) = @_;
-    my ($leap, $sec, $min, $hour, $year, $yday);
+    my ($leap, $ourleap, $sec, $min, $hour, $year, $yday);
     $ERROR = '';
 
     $ERROR .= "You must pass in a date to set me equal to" 
@@ -114,13 +130,51 @@ sub set_date {
 
     elsif(int $date) {
 	($year, $yday) = (localtime($date))[5,7];
-	$year += 1900;
+	$self->{year} = $year + 7364; #1900 + 5464
 	$self->{holiday} = 0; #assume this unless we find otherwise
 
+	$ourleap = 0;
+	$ourleap = 1 if ($year % 4 == 0) and ($year % 100 != 0);
+	$ourleap = 1 if $year % 400 == 0;
+
+	#Now for year adjustments since "except every 100 except every
+	#400 year rule applies to different years in the two calendars"
 	$leap = 0;
-	$leap = 1 if ($year % 4 == 0) and ($year % 100 != 0);
-	$leap = 1 if $year % 400 == 0;
-	$self->{year} = $year + 5464;
+	$year = $self->{year} % 400;  #don't need old value of $year anymore
+	if ($year == 300) {  #Shire calendar goes ahead one day in our Feb.
+	    if ((!$ourleap && $yday == 365) || ($ourleap && $yday == 366)) {
+		$yday = 1;
+		++$self->{year};
+	    }
+	} elsif (($year > 300) && ($year < 364)) { #shire ahead 1 day
+	    if ((!$ourleap && $yday == 365) || ($ourleap && $yday == 366)) {
+		$yday = 1;
+		++$self->{year};
+	    } else {
+		++$yday
+	    }
+	} elsif ($year == 164) { #calendars together after Overlithe
+	    ++$yday;
+	} elsif ((($year > 64) && ($year < 100)) || (($year > 164) && ($year < 200))) { #shire behind 1 day
+	    if ($yday == 1) {
+		--$self->{year};
+		$leap = 1 if ($self->{year} % 4 == 0) and ($self->{year} % 100 != 0);
+		$leap = 1 if $self->{year} % 400 == 0;
+		if ($leap) { $yday = 366; }
+		else { $yday = 365; }
+	    } else {
+		--$yday;
+	    }
+	} elsif (($year == 100) || ($year == 200)) { #equal after Feb 29
+	    if ($yday == 1) {
+		--$self->{year};
+		$yday = 365;
+	    } else {
+		--$yday;
+	    }
+	}
+	$leap = 1 if ($self->{year} % 4 == 0) and ($self->{year} % 100 != 0);
+	$leap = 1 if $self->{year} % 400 == 0;
 	if ($leap and $yday > 356) { ++$self->{year}; }
 	elsif ($yday > 355 and !$leap) { ++$self->{year}; }
 
@@ -132,11 +186,11 @@ sub set_date {
 	#leap year can now be ignored for the rest of this
 	#now check for any of the other holidays
 	unless ($self->{holiday}) {
-	    if ($yday == 356) {$self->{holiday} = 1;} #Yule 2-first day of new year
-	    elsif ($yday == 172) {$self->{holiday} = 2;} #Lithe 1
+	    if ($yday == 356) {$self->{holiday} = 1;} #2 Yule-first day of new year
+	    elsif ($yday == 172) {$self->{holiday} = 2;} #1 Lithe
 	    elsif ($yday == 173) {$self->{holiday} = 3;} #Midyear's day
-	    elsif ($yday == 174) {$self->{holiday} = 5;} #Lithe 2
-	    elsif ($yday == 355) {$self->{holiday} = 6;} #Yule 1
+	    elsif ($yday == 174) {$self->{holiday} = 5;} #2 Lithe
+	    elsif ($yday == 355) {$self->{holiday} = 6;} #1 Yule
 	} #end unless
 
 	#now compute the day of the week.  
@@ -146,7 +200,7 @@ sub set_date {
 	    $self->{weekday} = 0;
 	} #end if
 	else {
-	    $self->{weekday} = ($yday % 7) + 1;
+	    $self->{weekday} = (($yday + 2) % 7) + 1;
 	} #end else
 
 	#Now figure out the month and day of the month
@@ -156,7 +210,7 @@ sub set_date {
 	    $self->{monthday} = 0;
 	} #end if
 	else {
-	    --$yday; #ignore Yule 2
+	    --$yday; #ignore 2 Yule
 	    $yday -= 2 if $yday > 172; #ignore the Lithes (correct plural???)
 	    $yday += 9; #account for different start of year
 	    $yday -= 362 if ($yday > 361);		
@@ -183,7 +237,7 @@ caveats and error handling with that module apply to this method as well.
 
 sub time_in_seconds {
     my $self = shift;
-    my (@monthlen, $leap, $prevleap, $year, $month, $day);
+    my (@monthlen, $leap, $prevleap, $ourleap, $year, $month, $day, $modyear);
     $ERROR = '';
     @monthlen = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
 
@@ -195,13 +249,14 @@ sub time_in_seconds {
     } #end if
 
     $year = $self->{year} - 5464;
+    #both $leap and $prevleap refer to shire calendar
     $leap = 0;
-    $leap = 1 if (($year % 4 == 0) and ($year % 100 != 0));
-    $leap = 1 if ($year % 400 == 0);
+    $leap = 1 if (($self->{year} % 4 == 0) and ($self->{year} % 100 != 0));
+    $leap = 1 if ($self->{year} % 400 == 0);
     $prevleap = 0;
-    $prevleap = 1 if ((($year - 1) % 4 == 0)
-		      and (($year - 1) % 100 != 0));
-    $prevleap = 1 if (($year - 1) % 400 == 0);
+    $prevleap = 1 if ((($self->{year} - 1) % 4 == 0)
+		      and (($self->{year} - 1) % 100 != 0));
+    $prevleap = 1 if (($self->{year} - 1) % 400 == 0);
 
     #compute the year-day in our calendar from the shire one, and set the year
     if ($self->{holiday}) {
@@ -221,24 +276,46 @@ sub time_in_seconds {
 	$day += 3 if $day > 180; #Account for the Lithe and Midyear day
 	++$day if $leap and $day > 182; #Account for Overlithe
 	$day -= 8;  #Account for year starting on a different day
-	if ($day < 1) {
-	    --$year;
-	    if ($prevleap) { $day += 366; }
-	    else { $day += 365; }
-	} #end if ($day < 1)
     } #end else
 
-    #take leap years into account
-    $monthlen[1] = 29 if (($year % 4 == 0) and ($year %100 != 0)); 
+    #Now for adjustments for various years being off by a day
+    $modyear = $self->{year} % 400;
+    if (($modyear > 300) && ($modyear < 364)) {
+	--$day unless ($modyear == 301) && ($self->{holiday} == 1);
+    } elsif ((($modyear > 64) && ($modyear <= 100)) || (($modyear > 164) && ($modyear <= 200))) {
+	++$day;
+	$ourleap = 0;
+	$ourleap = 1 if (($year % 4 == 0) and ($year % 100 != 0));
+	$ourleap = 1 if ($year % 400 == 0);
+	if ($day == 367) {
+	    ++$year;
+	    $day = 1;
+	} elsif ($day == 366 and not $ourleap) {
+	    ++$year;
+	    $day = 1;
+	}
+    } elsif (($modyear == 101) || ($modyear == 201)) {
+	++$day if ($self->{holiday} == 1);
+    }
+
+    if ($day < 1) {
+	--$year;
+	if ($year % 4 != 0) { $day += 365; }
+	elsif (($year % 100 == 0) && ($year % 400 != 0)) { $day += 365; }
+	else { $day += 366; }
+    } #end if ($day < 1)
+
+    #take our leap years into account
+    $monthlen[1] = 29 if (($year % 4 == 0) and ($year %100 != 0));
     $monthlen[1] = 29 if ($year % 400 == 0);
-    
+
     #now convert the year-day to the month and month-day
     $month = 0;
     while ($day > $monthlen[$month]) {
 	$day -= $monthlen[$month];
 	++$month;
     } #end while ($day > $monthlen[$month])
-    
+
     return timelocal(0, 0, 0, $day, $month, $year);
 } #end sub time_in_seconds 
 
@@ -342,16 +419,16 @@ sub day {
     $holiday = $shiredate->holiday;
 
 If the day in question is a holiday, returns a string which holiday it is:
-"Yule 1", "Yule 2" (first day of the new year), "Lithe 1", "Midyear's day",
-"Overlithe", or "Lithe 2".  If the day is not a holiday, the null string
+"1 Yule", "2 Yule" (first day of the new year), "1 Lithe", "Midyear's day",
+"Overlithe", or "2 Lithe".  If the day is not a holiday, the null string
 is returned
 
 =cut
 
 sub holiday {
     my $self = shift;
-    my @holidays = ('', 'Yule 2', '1 Lithe', "Midyear's day", 'Overlithe',
-		    'Lithe 2', 'Yule 1');
+    my @holidays = ('', '2 Yule', '1 Lithe', "Midyear's day", 'Overlithe',
+		    '2 Lithe', '1 Yule');
     $ERROR = '';
     if (defined $self->{holiday}) {
 	return $holidays[$self->{holiday}];
@@ -508,7 +585,7 @@ sub on_date {
     # the day matches up
     $events{0} = { 3  => "Wedding of King Elessar and Arwen, 1419.\n"
 		   };
-    $events{1} = { 8  => "The Company of the Ring reaches Holland, 1419.\n",
+    $events{1} = { 8  => "The Company of the Ring reaches Hollin, 1419.\n",
 		   13 => "The Company of the Ring reaches the West-gate of Moria at nightfall, 1419.\n",
 		   14 => "The Company of the Ring spends the night in Moria hall 21, 1419.\n",
 		   15 => "The Bridge of Khazad-dum, and the fall of Gandalf, 1419.\n",
@@ -522,15 +599,15 @@ sub on_date {
 		   16 => "Company of the Ring says farewell to Lorien --\n" . 
 		       "Gollum observes departure, 1419.\n",
 		   17 => "Gwaihir the eagle bears Gandalf to Lorien, 1419.\n",
-		   23 => "The Company of the Ring pass the Argonath and camp at Parth Galen, 1419.\n" .
+		   25 => "The Company of the Ring pass the Argonath and camp at Parth Galen, 1419.\n" .
 		       "First battle of the Fords of Isen -- Theodred son of Theoden slain, 1419.\n",
-		   25 => "Breaking of the Fellowship, 1419.\n" .
+		   26 => "Breaking of the Fellowship, 1419.\n" .
 		       "Death of Boromir; his horn is heard in Minas Tirith, 1419.\n" .
 		       "Meriadoc and Peregrin captured by Orcs -- Aragorn pursues, 1419.\n" .
-		       "Eomer hears of the descent of the Orc-band from Emyn Muil, 1418.\n" .
+		       "Eomer hears of the descent of the Orc-band from Emyn Muil, 1419.\n" .
 		       "Frodo and Samwise enter the eastern Emyn Muil, 1419.\n",
 		   27 => "Aragorn reaches the west-cliff at sunrise, 1419.\n" .
-		       "Eomer sets out Eastfold against Theoden's orders to pursue the Orcs, 1419.\n",
+		       "Eomer sets out from Eastfold against Theoden's orders to pursue the Orcs, 1419.\n",
 		   28 => "Eomer overtakes the Orcs just outside of Fangorn Forest, 1419.\n",
 		   29 => "Meriodoc and Pippin escape and meet Treebeard, 1419.\n" .
 		       "The Rohirrim attack at sunrise and destroy the Orcs, 1419.\n" .
@@ -564,7 +641,7 @@ sub on_date {
 		   11 => "Gollum visits Shelob, 1419.\n" . 
 		       "Denethor sends Faramir to Osgiliath, 1419.\n" .
 		       "Eastern Rohan is invaded and Lorien assaulted, 1419.\n",
-		   12 => "Gollum leads Frodo in Shelob's lair, 1419.\n" .
+		   12 => "Gollum leads Frodo into Shelob's lair, 1419.\n" .
 		       "Ents defeat the invaders of Rohan, 1419.\n",
 		   13 => "Frodo captured by the Orcs of Cirith Ungol, 1419.\n" .
 		       "The Pelennor is overrun and Faramir is wounded, 1419.\n" .
@@ -591,7 +668,7 @@ sub on_date {
 		   27 => "Bard II and Thorin III Stonehelm drive the enemy from Dale, 1419.\n",
 		   28 => "Celeborn crosses the Anduin and begins destruction of Dol Guldur, 1419.\n"
 		   };
-    $events{4} = { 6  => "The mallorn trea flowers in the party field, 1420.\n",
+    $events{4} = { 6  => "The mallorn tree flowers in the party field, 1420.\n",
 	           8  => "Ring bearers are honored on the fields of Cormallen, 1419.\n",
 	           12 => "Gandalf arrives in Hobbiton, 1418\n"
 	           };
@@ -608,7 +685,7 @@ sub on_date {
     $events{8} = { 10 => "Funeral of King Theoden, 1419.\n"
 		   };
     $events{9} = { 18 => "Gandalf escapes from Orthanc in the early hours, 1418.\n",
-		   19 => "Gandalf comes to Edoras as a beggar, and is refused admintance, 1418",
+		   19 => "Gandalf comes to Edoras as a beggar, and is refused admittance, 1418",
 		   20 => "Gandalf gains entrance to Edoras.  Theoden commands him to go:\n" .
 		       "\"Take any horse, only be gone ere tomorrow is old\", 1418.\n",
 		   21 => "The hobbits return to Rivendell, 1419.\n",
@@ -617,7 +694,7 @@ sub on_date {
 		       "  they drive off the guard of Rangers, 1418.\n" .
 		       "Saruman comes to the Shire, 1419.\n",   
 		   23 => "Four Black Riders enter the shire before dawn.  The others pursue \n" .
-		       "the Rangers eastword and then return to watch the Greenway, 1418.\n" .
+		       "the Rangers eastward and then return to watch the Greenway, 1418.\n" .
 		       "A Black Rider comes to Hobbiton at nightfall, 1418.\n" . 
 		       "Frodo leaves Bag End, 1418.\n" .
 		       "Gandalf having tamed Shadowfax rides from Rohan, 1418.\n",
@@ -631,7 +708,7 @@ sub on_date {
             	   };
     $events{10} = { 3  => "Gandalf attacked at night on Weathertop, 1418.\n",
 		    5  => "Gandalf and the Hobbits leave Rivendell, 1419.\n",
-		    6  => "The camp under weathertop is attacked at night and Frodo is wounded, 1418.\n",
+		    6  => "The camp under Weathertop is attacked at night and Frodo is wounded, 1418.\n",
 		    11 => "Glorfindel drives the Black Riders off the Bridge of Mitheithel, 1418.\n",
 		    13 => "Frodo crosses the Bridge of Mitheithel, 1418.\n",
 		    18 => "Glorfindel finds Frodo at dusk, 1418.\n" . 
@@ -670,8 +747,6 @@ O'Brien from sending me this link).  I took this approximate as an exact
 and calculated back 6000 years from 1958 and set this as the start of the 
 4th age (1422).  Thus the fourth age begins in our B.C 4042.
 
-There is one issue that I have not finished yet with year calculation and it
-may change in the future (and opinions on it are welcome, as always).
 According to Appendix D of the Lord of the Rings, leap years in hobbit
 calendar are every 4 years unless its the turn of the century, in which
 case it's not a leap year.  Our calendar uses every 4 years unless it's 
@@ -686,30 +761,26 @@ then.  So instead, I have modified Tolkien's description of the hobbit
 calendar so that leap years occur once every 4 years unless it's 100
 years unless it's 400 years, so as it matches our calendar in that
 regard.  These 100 and 400 year intervals occur at different times in
-the two calendars, however.
-
-This final fact leads to the one known issue still in this module.
-Currently, the logic used here only works for years between 1937 and
-2035 (shire years 7401 to 7499).  This is due to the day offset at
-different times (in 400 year cycles) between the leap years of the different
-calendars.  The module still works for other dates and will provide valid
-comparisons, but the day it gives will be slightly off from what is actually
-the shire date unless you are between 1937 and 2035 (or 1537 and 1635, etc).
-I am planning on fixing this, but that must take a back seat for the moment 
-to my college classes and more practical projects.
+the two calendars, however.  Thus the last day of our year is sometimes
+7 Afteryule, sometimes 8, and sometimes 9.
 
 =head1 BIBLIOGRAPHY
 
 Tolkien, J. R. R. <i>Return of the King<i>.  New York: Houghton Mifflin Press,
 1955.
+
 http://www.glyphweb.com/arda/f/fourthage.html
 
 =head1 BUGS
 
-At present, does shire date reckoning is slightly off for years not between
-1937 and 2035 (shire reckoning 7401 to 7499).  See year calculation for more
-information.
+Epoch time.  Since epoch time was used as the base for this module, and the
+only way to currently set a date is from epoch time, it borks on values that
+epoch time doesn't support (currently values before 1902 or after 2037).  The
+module should automatically expand in available dates directly with epoch time
+support on your system.
 
 =cut
 
 1;
+
+# ex: set textwidth=72 :
